@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: server.c,v 1.133 2003/11/14 16:49:07 rjs3 Exp $
+ * $Id: server.c,v 1.141 2004/07/06 13:42:23 rjs3 Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -95,7 +95,7 @@ static int _sasl_checkpass(sasl_conn_t *conn,
 
 static mech_list_t *mechlist = NULL; /* global var which holds the list */
 
-static sasl_global_callbacks_t global_callbacks;
+sasl_global_callbacks_t global_callbacks;
 
 /* set the password for a user
  *  conn        -- SASL connection
@@ -126,13 +126,14 @@ int sasl_setpass(sasl_conn_t *conn,
 		 unsigned oldpasslen,
 		 unsigned flags)
 {
-    int result=SASL_OK, tmpresult;
+    int result = SASL_OK, tmpresult;
     sasl_server_conn_t *s_conn = (sasl_server_conn_t *) conn;
     const char *password_request[] = { SASL_AUX_PASSWORD_PROP, NULL };
     sasl_server_userdb_setpass_t *setpass_cb = NULL;
     void *context = NULL;
     int tried_setpass = 0;
     mechanism_t *m;
+    char *current_mech;
      
     if (!_sasl_server_active || !mechlist) return SASL_NOTINIT;
 
@@ -144,6 +145,20 @@ int sasl_setpass(sasl_conn_t *conn,
         || ((flags & SASL_SET_CREATE) && (flags & SASL_SET_DISABLE)))
 	PARAMERROR(conn);
 
+    /* Check that we have an active SASL mechanism */
+    if (sasl_getprop (conn,
+		      SASL_MECHNAME,
+		      (const void **) &current_mech) != SASL_OK) {
+	current_mech = NULL;
+    }
+
+    if ( (flags & SASL_SET_CURMECH_ONLY) &&
+	 (current_mech == NULL) ) {
+	sasl_seterror( conn, SASL_NOLOG,
+                  "No current SASL mechanism available");
+	RETURN(conn, SASL_BADPARAM);
+    }
+
     /* Do we want to store SASL_AUX_PASSWORD_PROP (plain text)?  and
      * Do we have an auxprop backend that can store properties?
      */
@@ -175,16 +190,19 @@ int sasl_setpass(sasl_conn_t *conn,
 	}
     }
 
+    /* We want to preserve the current value of result, so we use tmpresult below */
+
     /* call userdb callback function */
-    result = _sasl_getcallback(conn, SASL_CB_SERVER_USERDB_SETPASS,
+    tmpresult = _sasl_getcallback(conn, SASL_CB_SERVER_USERDB_SETPASS,
 			       &setpass_cb, &context);
-    if(result == SASL_OK && setpass_cb) {
+    if (tmpresult == SASL_OK && setpass_cb) {
 
 	tried_setpass++;
 
 	tmpresult = setpass_cb(conn, context, user, pass, passlen,
 			    s_conn->sparams->propctx, flags);
 	if(tmpresult != SASL_OK) {
+	    result = tmpresult;
 	    _sasl_log(conn, SASL_LOG_ERR,
 		      "setpass callback failed for %s: %z",
 		      user, tmpresult);
@@ -192,8 +210,6 @@ int sasl_setpass(sasl_conn_t *conn,
 	    _sasl_log(conn, SASL_LOG_NOTE,
 		      "setpass callback succeeded for %s", user);
 	}
-    } else {
-	result = SASL_OK;
     }
 
     /* now we let the mechanisms set their secrets */
@@ -203,6 +219,13 @@ int sasl_setpass(sasl_conn_t *conn,
 	    continue;
 	}
 
+	/* Invoke only one setpass for the currently selected mechanism,
+	   if SASL_SET_CURMECH_ONLY is specified */
+	if ((flags & SASL_SET_CURMECH_ONLY) &&
+	    (strcmp(current_mech, m->plug->mech_name) != 0)) {
+	    continue;
+	}
+
 	tried_setpass++;
 
 	tmpresult = m->plug->setpass(m->plug->glob_context,
@@ -273,6 +296,9 @@ static void server_dispose(sasl_conn_t *pconn)
   if (s_conn->sparams->propctx)
       prop_dispose(&s_conn->sparams->propctx);
 
+  if (s_conn->appname)
+      sasl_FREE(s_conn->appname);
+
   if (s_conn->user_realm)
       sasl_FREE(s_conn->user_realm);
 
@@ -354,7 +380,7 @@ int sasl_server_add_plugin(const char *plugname,
 	mech->version = version;
 
 	/* wheather this mech actually has any users in it's db */
-	mech->condition = result; /* SASL_OK or SASL_NOUSER */
+	mech->condition = result; /* SASL_OK, SASL_CONTINUE or SASL_NOUSER */
 
 	mech->next = mechlist->mech_list;
 	mechlist->mech_list = mech;
@@ -438,6 +464,12 @@ static int load_config(const sasl_callback_t *verifyfile_cb)
   size_t len;
   const sasl_callback_t *getpath_cb=NULL;
 
+  /* If appname was not provided, behave as if there is no config file 
+     (see also sasl_config_init() */
+  if (global_callbacks.appname == NULL) {
+      return SASL_CONTINUE;
+  }
+
   /* get the path to the plugins; for now the config file will reside there */
   getpath_cb=_sasl_find_getpath_callback( global_callbacks.callbacks );
   if (getpath_cb==NULL) return SASL_BADPARAM;
@@ -615,7 +647,8 @@ static int parse_mechlist_file(const char *mechlistfile)
 /* initialize server drivers, done once per process
  *  callbacks      -- callbacks for all server connections; must include
  *                    getopt callback
- *  appname        -- name of calling application (for lower level logging)
+ *  appname        -- name of calling application
+ *                    (for lower level logging and reading of the configuration file)
  * results:
  *  state          -- server state
  * returns:
@@ -643,8 +676,8 @@ int sasl_server_init(const sasl_callback_t *callbacks,
 	{ NULL, NULL }
     };
 
-    /* we require the appname to be non-null and short enough to be a path */
-    if (!appname || strlen(appname) >= PATH_MAX)
+    /* we require the appname (if present) to be short enough to be a path */
+    if (appname != NULL && strlen(appname) >= PATH_MAX)
 	return SASL_BADPARAM;
 
     if (_sasl_server_active) {
@@ -664,7 +697,12 @@ int sasl_server_init(const sasl_callback_t *callbacks,
 	return ret;
 
     global_callbacks.callbacks = callbacks;
-    global_callbacks.appname = appname;
+    
+    /* A shared library calling sasl_server_init will pass NULL as appname.
+       This should retain the original appname. */
+    if (appname != NULL) {
+        global_callbacks.appname = appname;
+    }
 
     /* If we fail now, we have to call server_done */
     _sasl_server_active = 1;
@@ -874,8 +912,21 @@ int sasl_server_new(const char *service,
   serverconn->sparams->service = (*pconn)->service;
   serverconn->sparams->servicelen = (unsigned) strlen((*pconn)->service);
 
-  serverconn->sparams->appname = global_callbacks.appname;
-  serverconn->sparams->applen = (unsigned) strlen(global_callbacks.appname);
+  if (global_callbacks.appname && global_callbacks.appname[0] != '\0') {
+    result = _sasl_strdup (global_callbacks.appname,
+			   &serverconn->appname,
+			   NULL);
+    if (result != SASL_OK) {
+      result = SASL_NOMEM;
+      goto done_error;
+    }
+    serverconn->sparams->appname = serverconn->appname;
+    serverconn->sparams->applen = (unsigned) strlen(serverconn->sparams->appname);
+  } else {
+    serverconn->appname = NULL;
+    serverconn->sparams->appname = NULL;
+    serverconn->sparams->applen = 0;
+  }
 
   serverconn->sparams->serverFQDN = (*pconn)->serverFQDN;
   serverconn->sparams->slen = (unsigned) strlen((*pconn)->serverFQDN);
@@ -932,17 +983,18 @@ static int mech_permitted(sasl_conn_t *conn,
 {
     sasl_server_conn_t *s_conn = (sasl_server_conn_t *)conn;
     const sasl_server_plug_t *plug;
+    int ret;
     int myflags;
     context_list_t *cur;
     sasl_getopt_t *getopt;
     void *context;
     sasl_ssf_t minssf = 0;
 
-    if(!conn) return 0;
+    if(!conn) return SASL_NOMECH;
 
     if(! mech || ! mech->plug) {
 	PARAMERROR(conn);
-	return 0;
+	return SASL_NOMECH;
     }
     
     plug = mech->plug;
@@ -969,7 +1021,7 @@ static int mech_permitted(sasl_conn_t *conn,
 		while (*mlist && isspace((int) *mlist)) mlist++;
 	    }
 
-	    if (!*mlist) return 0;  /* reached EOS -> not in our list */
+	    if (!*mlist) return SASL_NOMECH;  /* reached EOS -> not in our list */
 	}
     }
 
@@ -984,7 +1036,7 @@ static int mech_permitted(sasl_conn_t *conn,
     for(cur = s_conn->mech_contexts; cur; cur=cur->next) {
 	if(cur->mech == mech) {
 	    /* If it's not mech_avail'd, then stop now */
-	    if(!cur->context) return 0;
+	    if(!cur->context) return SASL_NOMECH;
 	    break;
 	}
     }
@@ -999,32 +1051,36 @@ static int mech_permitted(sasl_conn_t *conn,
     if (plug->max_ssf < minssf) {
 	sasl_seterror(conn, SASL_NOLOG,
 		      "mech %s is too weak", plug->mech_name);
-	return 0; /* too weak */
+	return SASL_TOOWEAK; /* too weak */
     }
 
     context = NULL;
     if(plug->mech_avail
-       && plug->mech_avail(plug->glob_context,
-			   s_conn->sparams, (void **)&context) != SASL_OK ) {
-	/* Mark this mech as no good for this connection */
-	cur = sasl_ALLOC(sizeof(context_list_t));
-	if(!cur) {
-	    MEMERROR(conn);
-	    return 0;
+       && (ret = plug->mech_avail(plug->glob_context,
+			   s_conn->sparams, (void **)&context)) != SASL_OK ) {
+	if(ret == SASL_NOMECH) {
+	    /* Mark this mech as no good for this connection */
+	    cur = sasl_ALLOC(sizeof(context_list_t));
+	    if(!cur) {
+		MEMERROR(conn);
+		return SASL_NOMECH;
+	    }
+	    cur->context = NULL;
+	    cur->mech = mech;
+	    cur->next = s_conn->mech_contexts;
+	    s_conn->mech_contexts = cur;
 	}
-	cur->context = NULL;
-	cur->mech = mech;
-	cur->next = s_conn->mech_contexts;
-	s_conn->mech_contexts = cur;
+	
+	/* SASL_NOTDONE might also get us here */
 
 	/* Error should be set by mech_avail call */
-	return 0;
+	return SASL_NOMECH;
     } else if(context) {
 	/* Save this context */
 	cur = sasl_ALLOC(sizeof(context_list_t));
 	if(!cur) {
 	    MEMERROR(conn);
-	    return 0;
+	    return SASL_NOMECH;
 	}
 	cur->context = context;
 	cur->mech = mech;
@@ -1035,20 +1091,20 @@ static int mech_permitted(sasl_conn_t *conn,
     /* Generic mechanism */
     if (plug->max_ssf < minssf) {
 	sasl_seterror(conn, SASL_NOLOG, "too weak");
-	return 0; /* too weak */
+	return SASL_TOOWEAK; /* too weak */
     }
 
     /* if there are no users in the secrets database we can't use this 
        mechanism */
     if (mech->condition == SASL_NOUSER) {
 	sasl_seterror(conn, 0, "no users in secrets db");
-	return 0;
+	return SASL_NOMECH;
     }
 
     /* Can it meet our features? */
     if ((conn->flags & SASL_NEED_PROXY) &&
 	!(plug->features & SASL_FEAT_ALLOWS_PROXY)) {
-	return 0;
+	return SASL_NOMECH;
     }
     
     /* security properties---if there are any flags that differ and are
@@ -1064,10 +1120,10 @@ static int mech_permitted(sasl_conn_t *conn,
     }
 
     /* do we want to special case SASL_SEC_PASS_CREDENTIALS? nah.. */
-    if (((myflags ^ plug->security_flags) & myflags) != 0) {
+    if ((myflags &= (myflags ^ plug->security_flags)) != 0) {
 	sasl_seterror(conn, SASL_NOLOG,
 		      "security flags do not match required");
-	return 0;
+	return (myflags & SASL_SEC_NOPLAINTEXT) ? SASL_ENCRYPT : SASL_NOMECH;
     }
 
     /* Check Features */
@@ -1076,10 +1132,10 @@ static int mech_permitted(sasl_conn_t *conn,
 	sasl_seterror(conn, 0,
 		      "mech %s requires unprovided secret facility",
 		      plug->mech_name);
-	return 0;
+	return SASL_NOMECH;
     }
 
-    return 1;
+    return SASL_OK;
 }
 
 /*
@@ -1168,8 +1224,7 @@ int sasl_server_start(sasl_conn_t *conn,
     }
 
     /* Make sure that we're willing to use this mech */
-    if (! mech_permitted(conn, m)) {
-	result = SASL_NOMECH;
+    if ((result = mech_permitted(conn, m)) != SASL_OK) {
 	goto done;
     }
 
@@ -1477,11 +1532,11 @@ int _sasl_server_listmech(sasl_conn_t *conn,
   /* make list */
   for (lup = 0; lup < mechlist->mech_length; lup++) {
       /* currently, we don't use the "user" parameter for anything */
-      if (mech_permitted(conn, listptr)) {
+      if (mech_permitted(conn, listptr) == SASL_OK) {
 	  if (pcount != NULL)
 	      (*pcount)++;
 
-	  /* print seperator */
+	  /* print separator */
 	  if (flag) {
 	      strcat(conn->mechlist_buf, mysep);
 	  } else {
@@ -1815,6 +1870,10 @@ int sasl_checkapop(sasl_conn_t *conn,
         RETURN(conn, result);
     }
 
+    /* erase the plaintext password */
+    s_conn->sparams->utils->prop_erase(s_conn->sparams->propctx,
+				       password_request[0]);
+
     /* Cannonify it */
     result = _sasl_canon_user(conn, user, user_len,
 	                      SASL_CU_AUTHID | SASL_CU_AUTHZID,
