@@ -1,7 +1,7 @@
 /* testsuite.c -- Stress the library a little
  * Rob Siemborski
  * Tim Martin
- * $Id: testsuite.c,v 1.36 2004/02/20 23:52:58 rjs3 Exp $
+ * $Id: testsuite.c,v 1.39 2004/10/24 20:05:22 shadow Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -97,10 +97,10 @@ char myhostname[1024+1];
 #define REALLY_LONG_LENGTH  32000
 #define REALLY_LONG_BACKOFF  2000
 
-const char *username = "rjs3";
+const char *username = "shadow";
 const char *nonexistant_username = "ABCDEFGHIJ";
-const char *authname = "rjs3";
-const char *proxyasname = "rjs3proxy";
+const char *authname = "shadow";
+const char *proxyasname = "shadowproxy";
 const char *password = "1234";
 const char *cu_plugin = "INTERNAL";
 char other_result[1024];
@@ -603,10 +603,6 @@ void test_init(void)
     sasl_done();
     if(mem_stat() != SASL_OK) fatal("memory error after long appname test");
 
-    /* try passing NULL name */
-    result = sasl_server_init(emptysasl_cb, NULL);
-    if (result == SASL_OK) fatal("Allowed null name to sasl_server_init");
-
     /* this calls sasl_done when it wasn't inited */
     sasl_done();
     if(mem_stat() != SASL_OK) fatal("memory error after null appname test");
@@ -625,12 +621,13 @@ void test_init(void)
     sasl_done();
     if(mem_stat() != SASL_OK) fatal("memory error after client test");
 
+#if defined(DO_DLOPEN) && (defined(PIC) || (!defined(PIC) && defined(TRY_DLOPEN_WHEN_STATIC)))
     /* try giving it an invalid path for where the plugins are */
     result = sasl_server_init(withbadpathsasl_cb, NULL);
-
     if (result==SASL_OK) fatal("Allowed invalid path");
     sasl_done();
     if(mem_stat() != SASL_OK) fatal("memory error after bad path test");
+#endif
 
     /* and the client - xxx is this necessary?*/
 #if 0
@@ -2862,7 +2859,7 @@ void notes(void)
     printf("-For both KERBEROS_V4 and GSSAPI you must have non-expired tickets\n");
     printf("-For OTP (w/OPIE) must be able to read/write opiekeys (/etc/opiekeys)\n");
     printf("-For OTP you must have a non-expired secret\n");
-    printf("-Must be able to read sasldb, which needs to be setup with a.\n");
+    printf("-Must be able to read sasldb, which needs to be setup with a\n");
     printf(" username and a password (see top of testsuite.c)\n");
     printf("\n\n");
 }
