@@ -2,7 +2,7 @@
  * Ryan Troll
  */
 /* 
- * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2004 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -44,29 +44,44 @@
 #ifndef CONFIG_H
 #define CONFIG_H
 
-#include <windows.h>
+#include <stddef.h>
+
+/* winsock2 includes windows.h.
+   Note that we can't include both winsock.h and winsock2.h as
+   they conflict */
+#include <winsock2.h>
 
 /* Our package */
 #define PACKAGE "cyrus-sasl"
 
 /* Our version */
-#define VERSION "2.1.15"
+#define VERSION "2.1.20"
+
+/* Visual Studio supports prototypes */
+#define PROTOTYPES     1
+
+#ifndef HAVE_CADDR_T
+#ifndef	caddr_t
+typedef unsigned char   *caddr_t;
+#define HAVE_CADDR_T	1
+#endif
+#endif
+
+#ifndef _INTPTR_T_DEFINED
+
+#ifdef  _WIN64
+typedef __int64             intptr_t;
+#else
+typedef int		    intptr_t;
+#endif
+
+#endif
 
 /* Registry key that contains the locations of the plugins */
 #define SASL_ROOT_KEY "SOFTWARE\\Carnegie Mellon\\Project Cyrus\\SASL Library"
 #define SASL_KEY_V1 SASL_ROOT_KEY "\\Available Plugins"
 #define SASL_PATH_SUBKEY "SearchPath"
 
-/* We only want minimal server functionality.  Cripple the server functionality when necessary to get
- * things to compile.
- *
- * Currently only cripples PLAIN.
- */
-#define SASL_MINIMAL_SERVER 1
-
-/* DB Type */
-#undef SASL_DB_TYPE
-
 /* : This should probably be replaced with a call to a function
    : that gets the proper value from Registry */
 #define SASL_DB_PATH "c:\\CMU\\sasldb2"
@@ -82,12 +97,12 @@
 #define STATIC_DIGESTMD5 1
 #define STATIC_GSSAPIV2 1
 /* #undef STATIC_KERBEROS4 */
-/* #undef STATIC_LOGIN */
+#define STATIC_LOGIN 1
 /* #undef STATIC_MYSQL */
 /* #undef STATIC_OTP */
 #define STATIC_PLAIN 1
 #define STATIC_SASLDB 1
-/* #undef STATIC_SRP */
+#define STATIC_SRP 1
 
 /* ------------------------------------------------------------ */
 
@@ -96,7 +111,7 @@
 #define HAVE_MEMCPY 1
 
 #define SASL_PATH_ENV_VAR "SASL_PATH"
-#define PLUGINDIR "C:\\SASL-PLUGINS"
+#define PLUGINDIR "C:\\CMU\\bin\\sasl2"
 
 /* Windows calls these functions something else
  */
@@ -112,7 +127,6 @@
 #define L_DEFAULT_GUARD (0)
 #define I_DEFAULT_GUARD (0)
 #define I(foo)
-//#define VL(foo)
 #define VL(foo)  printf foo;
 #define VLP(foo,bar)
 
@@ -124,7 +138,7 @@
 typedef int socklen_t;
 #endif /* HAVE_SOCKLEN_T */
 
-#ifndef HAVE_STRUCT_SOCKADDR_STORAGE
+#if !defined(HAVE_STRUCT_SOCKADDR_STORAGE) && !defined(_SS_MAXSIZE)
 #define	_SS_MAXSIZE	128	/* Implementation specific max size */
 #define	_SS_PADSIZE	(_SS_MAXSIZE - sizeof (struct sockaddr))
 
@@ -148,10 +162,15 @@ struct sockaddr_storage {
 #include "gai.h"
 #endif
 
-#ifndef	NI_WITHSCOPEID
-#define	NI_WITHSCOPEID	0
-#endif
-
 #include <time.h>
 
+typedef int ssize_t;
+
+#define HIER_DELIMITER '\\'
+
+#ifndef sleep
+#define sleep(seconds)		plug_sleep(seconds)
+unsigned int plug_sleep(unsigned int seconds);
+#endif
+
 #endif /* CONFIG_H */
